package com.dayou.controller;

import com.dayou.dto.AssetsComplaintSelectDTO;
import com.dayou.vo.AssetsComplaintVO;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestMapping;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.core.metadata.IPage;
import org.springframework.web.bind.annotation.RestController;
import com.dayou.controller.BaseController;
import com.dayou.service.IAssetsComplaintService;
import com.dayou.entity.AssetsComplaint;
import com.dayou.common.RestResponse;
import org.springframework.web.bind.annotation.*;
import com.dayou.utils.ConvertUtil;
import com.dayou.utils.HttpKit;
import com.dayou.exception.ErrorCode;

import java.util.Date;
import java.util.List;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import org.springframework.http.MediaType;
import org.springframework.web.multipart.MultipartFile;

/**
 * 资产业务项目投诉表
 *
 * @author GouGengquan
 * @since 2025-07-16
 */
@RestController
@RequestMapping("assetsComplaint")
@Slf4j
public class AssetsComplaintController extends BaseController {
    @Autowired
    private IAssetsComplaintService assetsComplaintService;

    /**
     * 查询投诉信息分页
     *
     * @param dto 查询参数
     * @return List<AssetsComplaintVO>
     */
    @GetMapping("/page")
    public RestResponse<Page<AssetsComplaintVO>> page(Page page, AssetsComplaintSelectDTO dto) {
        Page<AssetsComplaintVO> pages = assetsComplaintService.selectPage(page, dto);
        return RestResponse.data(pages);
    }

    /**
     * 查询投诉信息集合
     *
     * @param dto 查询参数
     * @return List<AssetsComplaintVO>
     */
    @GetMapping("/list")
    public RestResponse<List<AssetsComplaintVO>> listData(AssetsComplaintSelectDTO dto) {
        return RestResponse.data(assetsComplaintService.listData(dto));
    }

    /**
     * 撤销投诉
     */
    @PutMapping("/cancel/{id}")
    public RestResponse<Boolean> cancelComplaint(@PathVariable Long id) {
        return RestResponse.data(assetsComplaintService.cancelComplaint(id));
    }

    /**
     * 确认投诉
     */
    @PutMapping("/confirm/{id}")
    public RestResponse<Boolean> confirmComplaint(@PathVariable Long id) {
        return RestResponse.data(assetsComplaintService.confirmComplaint(id));
    }

    /**
     * 资产业务项目投诉表详情
     */
    @GetMapping("/detail/{id}")
    public RestResponse<AssetsComplaint> detail(@PathVariable Long id) {
        AssetsComplaint xAssetsComplaint = assetsComplaintService.detail(id);
        return RestResponse.data(xAssetsComplaint);
    }

    /**
     * 资产业务项目投诉表新增
     */
    @PostMapping("/save")
    public RestResponse<Boolean> save(@RequestBody AssetsComplaint assetsComplaint) {
        Boolean ret = assetsComplaintService.add(assetsComplaint);
        return RestResponse.data(ret);
    }

    /**
     * 资产业务项目投诉表更新
     */
    @PutMapping("/update")
    public RestResponse<Boolean> update(@RequestBody AssetsComplaint assetsComplaint) {
        Boolean ret = assetsComplaintService.update(assetsComplaint);
        return RestResponse.data(ret);
    }

    /**
     * 资产业务项目投诉表删除
     */
    @DeleteMapping("/delete/{id}")
    public RestResponse<Boolean> delete(@PathVariable Long id) {
        Boolean ret = assetsComplaintService.delete(id);
        return RestResponse.data(ret);
    }


}

