package com.dayou.controller;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.dayou.common.RestResponse;
import com.dayou.dto.MajorAppraiserUnitDTO;
import com.dayou.service.IMajorAppraiserUnitService;
import com.dayou.vo.MajorOrderStatEvaluateUnitVO;
import com.dayou.vo.UserRateVO;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;
import java.io.IOException;
import java.util.List;

/**
 * 大中型估价人员计件数(人力资源部统计)
 *
 * @author wucl
 * @since 2025-06-26
 */
@RestController
@RequestMapping("majorAppraiserUnit")
@Slf4j
public class MajorAppraiserUnitController extends BaseController {

    @Autowired
    private IMajorAppraiserUnitService majorAppraiserUnitService;

    /**
     * 分配计件数
     * @param majorAppraiserUnitDTO
     * @return
     */
    @PostMapping("")
    public RestResponse<Boolean> save(@RequestBody @Valid MajorAppraiserUnitDTO majorAppraiserUnitDTO) {
        Boolean ret = majorAppraiserUnitService.saveUnit(majorAppraiserUnitDTO);
        return RestResponse.success(ret);
    }

    /**
     * 查询计件数
     * @param id
     * @return
     */
    @GetMapping("/{id}")
    public RestResponse<MajorAppraiserUnitDTO> select(@PathVariable("id") Long id){
        MajorAppraiserUnitDTO ret = majorAppraiserUnitService.select(id);
        return RestResponse.success(ret);
    }

    /**
     *  项目计件数统计
     * @param unit
     * @param page
     * @return
     */
    @GetMapping("/order/stat")
    public RestResponse<Page<MajorOrderStatEvaluateUnitVO>> getOrderStat(MajorOrderStatEvaluateUnitVO unit, Page page){
        Page<MajorOrderStatEvaluateUnitVO> ret = majorAppraiserUnitService.getOrderStat(page,unit);
        return RestResponse.success(ret);
    }

    /**
     *  项目计件数统计导出
     * @param unit
     * @return
     */
    @GetMapping("/orderUnitExport")
    public void exportOrderExport(MajorOrderStatEvaluateUnitVO unit, HttpServletResponse response) throws IOException {
        List<MajorOrderStatEvaluateUnitVO> ret = majorAppraiserUnitService.exportOrderExport(unit);
        exportPlus(response,"项目计件数",ret, MajorOrderStatEvaluateUnitVO.class);
    }

    /**
     *  人员计件数统计
     * @param rate
     * @param page
     * @return
     */
    @GetMapping("/user/stat")
    public RestResponse<Page<UserRateVO>> getUserStat(UserRateVO rate, Page page){
        Page<UserRateVO> ret = majorAppraiserUnitService.getUserStat(page,rate);
        return RestResponse.success(ret);
    }


    /**
     *  人员计件数统计导出
     * @param rate
     * @return
     */
    @GetMapping("/userUnitExport")
    public void exportUserExport(UserRateVO rate, HttpServletResponse response) throws IOException {
        List<UserRateVO> ret = majorAppraiserUnitService.exportUserExport(rate);
        exportPlus(response,"人员计件数",ret, UserRateVO.class);
    }

    /**
     *  人员计件数详情列表
     * @param unit
     * @param page
     * @return
     */
    @GetMapping("/user/detail")
    public RestResponse<Page<MajorOrderStatEvaluateUnitVO>> getUserDetail(MajorOrderStatEvaluateUnitVO unit, Page page){
        Page<MajorOrderStatEvaluateUnitVO> ret = majorAppraiserUnitService.getUserDetail(page,unit);
        return RestResponse.success(ret);
    }
}
