package com.dayou.controller;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.dayou.annotation.IgnoreAuth;
import com.dayou.common.RestResponse;
import com.dayou.dto.PersonalFacePriceQueryDTO;
import com.dayou.entity.PersonalFacePrice;
import com.dayou.service.IPersonalFacePriceService;
import com.dayou.vo.DayouPersonalPriceVO;
import com.dayou.vo.ExternalFacePriceVO;
import lombok.extern.slf4j.Slf4j;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.io.ClassPathResource;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.util.List;

/**
 * 个贷口估
 */
@RestController
@RequestMapping("personalFacePrice")
@Slf4j
public class PersonalFacePriceController extends BaseController {

    @Autowired
    private IPersonalFacePriceService personalFacePriceService;

    /**
     * 查询大友个贷历史评估价格(地址)
     * @param facePriceQuery
     * @return
     */
    @PostMapping("/dy/location/price")
    public RestResponse<List<DayouPersonalPriceVO>> personalPriceByLocation(@RequestBody PersonalFacePriceQueryDTO facePriceQuery) {
        List<DayouPersonalPriceVO> ret = personalFacePriceService.personalPriceByLocation(facePriceQuery);
        return RestResponse.success(ret);
    }

    /**
     * 查询大友个贷历史评估价格(小区名字)
     * @param facePriceQuery
     * @return
     */
    @PostMapping("/dy/community/price")
    public RestResponse<List<DayouPersonalPriceVO>> personalPriceByCommunity(@RequestBody PersonalFacePriceQueryDTO facePriceQuery) {
        List<DayouPersonalPriceVO> ret = personalFacePriceService.personalPriceByCommunity(facePriceQuery);
        return RestResponse.success(ret);
    }

    /**
     * 查询外部数据价格（地址）
     * @param facePriceQuery
     * @return
     */
    @PostMapping("/external/location/price")
    public RestResponse<ExternalFacePriceVO> externalPriceByLocation(@RequestBody PersonalFacePriceQueryDTO facePriceQuery) {
        ExternalFacePriceVO ret = personalFacePriceService.externalPriceByLocation(facePriceQuery);
        return RestResponse.success(ret);
    }

    /**
     * 查询外部数据价格（小区名字）
     * @param facePriceQuery
     * @return
     */
    @PostMapping("/external/community/price")
    public RestResponse<ExternalFacePriceVO> externalPersonalPriceByCommunity(@RequestBody PersonalFacePriceQueryDTO facePriceQuery) {
        ExternalFacePriceVO ret = personalFacePriceService.externalPersonalPriceByCommunity(facePriceQuery);
        return RestResponse.success(ret);
    }

    /**
     * 执行口估
     * @param facePrice
     * @return
     */
    @PostMapping("")
    public RestResponse<Boolean> doFacePrice(@RequestBody PersonalFacePrice facePrice) {
        Boolean ret = personalFacePriceService.doFacePrice(facePrice);
        return RestResponse.success(ret);
    }

    /**
     * 分页查询口估执行记录
     * @param facePrice
     * @param page
     * @return
     */
    @GetMapping("")
    public RestResponse<IPage<PersonalFacePrice>> getPage(PersonalFacePrice facePrice, Page page){
        IPage<PersonalFacePrice> ret = personalFacePriceService.getPage(page,facePrice);
        return RestResponse.success(ret);
    }

    /**
     * 个贷批量口估模版下载
     * @param request
     * @param response
     */
    @IgnoreAuth
    @GetMapping("/download/temp")
    public void downloadPersonalFacePriceTemp(HttpServletRequest request, HttpServletResponse response){
        try {
            XSSFWorkbook workbook = new XSSFWorkbook(new ClassPathResource("xlsx/personal/personal_face_price_batch_temp.xlsx").getInputStream());
            writeXss2Response(request,response,workbook,"批量询价模版.xlsx");
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
    }

    /**
     * 个贷批量口估上传
     * @param file
     * @return
     */
    @IgnoreAuth
    @PostMapping("/upload/temp")
    public RestResponse<String> uploadPersonalFacePriceTemp(@RequestPart("file") MultipartFile file,@RequestParam("limit") String limit,HttpServletResponse response) throws IOException {
        String url = personalFacePriceService.uploadPersonalFacePriceTemp(file,limit);
        return RestResponse.data(url);
    }


}
