package com.dayou.service.impl;

import com.dayou.common.BaseEntity;
import com.dayou.dto.AssetsComplaintSelectDTO;
import com.dayou.entity.AssetsComplaint;
import com.dayou.mapper.AssetsComplaintMapper;
import com.dayou.service.IAssetsComplaintService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.dayou.utils.LoginContext;
import com.dayou.vo.AssetsComplaintVO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.dayou.utils.ExcelUtil;
import org.apache.poi.ss.usermodel.Cell;
import org.apache.poi.ss.usermodel.CellStyle;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.springframework.web.multipart.MultipartFile;

import java.util.Date;
import java.util.List;
import java.util.ArrayList;

import org.springframework.transaction.annotation.Transactional;
import com.dayou.enums.BatchTaskTypeEnum;

/**
 * <p>
 * 资产业务项目投诉表 服务实现类
 * </p>
 *
 * @author GouGengquan
 * @since 2025-07-16
 */
@Service
public class AssetsComplaintServiceImpl extends ServiceImpl<AssetsComplaintMapper, AssetsComplaint> implements IAssetsComplaintService {

    @Autowired
    private AssetsComplaintMapper assetsComplaintMapper;

    /**
     * 查询投诉信息分页
     *
     * @param dto 查询参数
     * @return List<AssetsComplaintVO>
     */
    @Override
    public Page<AssetsComplaintVO> selectPage(Page page, AssetsComplaintSelectDTO dto) {
        return assetsComplaintMapper.pageData(page, dto);
    }

    /**
     * 查询投诉信息集合
     *
     * @param dto 查询参数
     * @return List<AssetsComplaintVO>
     */
    @Override
    public List<AssetsComplaintVO> listData(AssetsComplaintSelectDTO dto) {
        return assetsComplaintMapper.listData(dto);
    }

    /**
     * 撤销投诉
     *
     * @param id 主键id
     * @return Boolean
     */
    @Override
    public Boolean cancelComplaint(Long id) {
        return this.update(new LambdaUpdateWrapper<AssetsComplaint>()
                .set(AssetsComplaint::getState, "CANCEL")
                .set(AssetsComplaint::getCheckerId, LoginContext.getCurrentUserId())
                .set(AssetsComplaint::getCheckDate, new Date())
                .eq(BaseEntity::getId, id)
                .eq(BaseEntity::getDeleted, false));
    }

    /**
     * 确认投诉
     *
     * @param id 主键id
     * @return Boolean
     */
    @Override
    public Boolean confirmComplaint(Long id) {
        return this.update(new LambdaUpdateWrapper<AssetsComplaint>()
                .set(AssetsComplaint::getState, "CONFIRM")
                .set(AssetsComplaint::getCheckerId, LoginContext.getCurrentUserId())
                .set(AssetsComplaint::getCheckDate, new Date())
                .eq(BaseEntity::getId, id)
                .eq(BaseEntity::getDeleted, false));
    }

    @Override
    public AssetsComplaint detail(Long id) {
        return this.getById(id);
    }

    /**
     * 新增投诉
     * @param assetsComplaint 投诉信息
     * @return Boolean
     */
    @Override
    public Boolean add(AssetsComplaint assetsComplaint) {
        assetsComplaint.setComplainantId(LoginContext.getCurrentUserId());
        return this.save(assetsComplaint);
    }

    @Override
    public Boolean update(AssetsComplaint assetsComplaint) {
        return this.updateById(assetsComplaint);
    }

    @Override
    public Boolean delete(Long id) {
        //逻辑删除
        return this.removeById(id);
    }
}
