package com.dayou.service.impl;

import cn.hutool.core.collection.CollectionUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.dayou.common.BaseEntity;
import com.dayou.dto.MajorAppraiserUnitDTO;
import com.dayou.entity.Major;
import com.dayou.entity.MajorAppraiserUnit;
import com.dayou.exception.ErrorCode;
import com.dayou.mapper.MajorAppraiserUnitMapper;
import com.dayou.service.IMajorAppraiserUnitService;
import com.dayou.service.IMajorService;
import com.dayou.vo.MajorOrderStatEvaluateUnitVO;
import com.dayou.vo.UserRateVO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.util.Collections;
import java.util.List;

@Service
public class MajorAppraiserUnitServiceImpl extends ServiceImpl<MajorAppraiserUnitMapper, MajorAppraiserUnit> implements IMajorAppraiserUnitService {

    @Autowired
    private IMajorService majorService;

    @Autowired
    private MajorAppraiserUnitMapper majorAppraiserUnitMapper;

    @Override
    public Boolean saveUnit(MajorAppraiserUnitDTO majorAppraiserUnitDTO) {

        Long majorId = majorAppraiserUnitDTO.getMajorId();

        Major major = majorService.getOne(new LambdaQueryWrapper<Major>().select(Major::getEvaluateUnit, BaseEntity::getId).eq(BaseEntity::getId, majorId).eq(BaseEntity::getDeleted, false));

        if (major.getEvaluateUnit() == null || major.getEvaluateUnit().compareTo(BigDecimal.ZERO)==0) {
            ErrorCode.throwBusinessException(ErrorCode.CUSTOM_ERROR,"请先添加估价对象后再分配计件数。");
        }


        this.remove(new LambdaQueryWrapper<MajorAppraiserUnit>().eq(MajorAppraiserUnit::getMajorId,majorId));

        List<MajorAppraiserUnit> userUnit = majorAppraiserUnitDTO.getUserUnit();


        BigDecimal sum = BigDecimal.ZERO;
        if (CollectionUtil.isNotEmpty(userUnit)){
            for (MajorAppraiserUnit unit : userUnit){
                unit.setMajorId(majorId);
                sum = sum.add(unit.getUnitRate());
            }
        }

        if (major.getEvaluateUnit().compareTo(sum)!=0){
            ErrorCode.throwBusinessException(ErrorCode.CUSTOM_ERROR,"分配之和必须等于总计件数值。");
        }
        return this.saveBatch(userUnit);
    }

    @Override
    public MajorAppraiserUnitDTO select(Long id) {
        List<MajorAppraiserUnit> list = this.list(new LambdaQueryWrapper<MajorAppraiserUnit>().eq(MajorAppraiserUnit::getMajorId, id));
        MajorAppraiserUnitDTO majorAppraiserUnitDTO = new MajorAppraiserUnitDTO();
        majorAppraiserUnitDTO.setMajorId(id);
        majorAppraiserUnitDTO.setUserUnit(list);
        return majorAppraiserUnitDTO;
    }

    @Override
    public Page<MajorOrderStatEvaluateUnitVO> getOrderStat(Page page, MajorOrderStatEvaluateUnitVO unit) {
        return majorAppraiserUnitMapper.getOrderStat(page,unit);
    }

    @Override
    public List<MajorOrderStatEvaluateUnitVO> exportOrderExport(MajorOrderStatEvaluateUnitVO unit) {
        List<MajorOrderStatEvaluateUnitVO> list = majorAppraiserUnitMapper.exportOrderExport(unit);
        list.stream().forEach(x->{
            x.setAllotUnitStr(x.getAllotUnitStrs());
        });
        return list;
    }

    @Override
    public Page<UserRateVO> getUserStat(Page page, UserRateVO rate) {
        return majorAppraiserUnitMapper.getUserStat(page,rate);
    }

    @Override
    public List<UserRateVO> exportUserExport(UserRateVO rate) {
        return majorAppraiserUnitMapper.getUserStatExport(rate);
    }

    @Override
    public Page<MajorOrderStatEvaluateUnitVO> getUserDetail(Page page, MajorOrderStatEvaluateUnitVO unit) {
        return majorAppraiserUnitMapper.getUserDetail(page,unit);
    }
}
