package com.dayou.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.dayou.dto.AddressDTO;
import com.dayou.dto.PersonalFacePriceBatchDTO;
import com.dayou.dto.PersonalFacePriceQueryDTO;
import com.dayou.entity.PersonalFacePrice;
import com.dayou.exception.ErrorCode;
import com.dayou.mapper.PersonalFacePriceMapper;
import com.dayou.service.IExternalDyDataBaseService;
import com.dayou.service.IGlobalConfigService;
import com.dayou.service.IPersonalFacePriceService;
import com.dayou.utils.*;
import com.dayou.vo.DayouPersonalPriceVO;
import com.dayou.vo.ExternalFacePriceVO;
import com.dayou.vo.ExternalPersonalVO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.net.URLEncoder;
import java.time.LocalDate;
import java.util.*;
import java.util.stream.Collectors;

import static com.dayou.enums.GlobalConfigEnum.SYS;
import static com.dayou.enums.GlobalConfigEnum.Sys.*;

@Service
public class PersonalFacePriceServiceImpl extends ServiceImpl<PersonalFacePriceMapper, PersonalFacePrice> implements IPersonalFacePriceService {

    @Autowired
    private PersonalFacePriceMapper personalFacePriceMapper;

    @Autowired
    private IExternalDyDataBaseService externalDyDataBaseService;

    @Autowired
    private IGlobalConfigService globalConfigService;

    @Override
    public List<DayouPersonalPriceVO> personalPriceByLocation(PersonalFacePriceQueryDTO facePriceQuery) {
        //获取配置的价值点范围
        Date startDate = DateUtils.facePriceLimitDate(facePriceQuery.getLimit());
        //查询策略1：全地址查询
        List<DayouPersonalPriceVO> byRoadNumberResult = new ArrayList<>();
        List<DayouPersonalPriceVO> facePricesByRoadNumber = new ArrayList<>();
        String location = facePriceQuery.getQueryTarget();
        AddressDTO targetAddress = AddressUtil.parseAddress(location);
        String roadNumberAddress = AddressUtil.getRoadNumberAddress(location);
        if (StrUtil.isNotBlank(roadNumberAddress)) {
            byRoadNumberResult = personalFacePriceMapper.queryByFuzzyAddress(roadNumberAddress,startDate,facePriceQuery.getAcreageMin(),facePriceQuery.getAcreageMax());
            facePricesByRoadNumber = personalFacePriceMapper.queryFromFacePriceHis(roadNumberAddress,startDate,facePriceQuery.getAcreageMin(),facePriceQuery.getAcreageMax());
            facePricesByRoadNumber.stream().forEach(x->x.setMatching("全地址匹配(口估)"));
            byRoadNumberResult.stream().forEach(x->x.setMatching("全地址匹配"));
        }

        //查询策略2：路名路号查询
        List<DayouPersonalPriceVO> byRoadNameNumberResult = new ArrayList<>();
        List<DayouPersonalPriceVO> facePricesByRoadNameNumber = new ArrayList<>();
        String roadAndRoadNumber = AddressUtil.getRoadAndRoadNumber(location);
        if (StrUtil.isNotBlank(roadAndRoadNumber)){
            byRoadNameNumberResult = personalFacePriceMapper.queryByFuzzyAddress(roadAndRoadNumber,startDate,facePriceQuery.getAcreageMin(),facePriceQuery.getAcreageMax());
            filterByCityAndDistrict(byRoadNameNumberResult,targetAddress);
            facePricesByRoadNameNumber = personalFacePriceMapper.queryFromFacePriceHis(roadNumberAddress,startDate,facePriceQuery.getAcreageMin(),facePriceQuery.getAcreageMax());
            facePricesByRoadNameNumber.stream().forEach(x->x.setMatching("路名路号匹配(口估)"));
            byRoadNameNumberResult.stream().forEach(x->x.setMatching("路名路号匹配"));
        }

        //查询策略3：尝试用小区名字查询
        List<DayouPersonalPriceVO> byCommunityNameResult = new ArrayList<>();
        List<DayouPersonalPriceVO> facePriceByCommunity = new ArrayList<>();
        String communityNameFromAddress = AddressUtil.getCommunityNameFromAddress(location);
        if (StrUtil.isNotBlank(communityNameFromAddress)){
            byCommunityNameResult = personalFacePriceMapper.queryByFuzzyAddress(communityNameFromAddress,startDate,facePriceQuery.getAcreageMin(),facePriceQuery.getAcreageMax());
            facePriceByCommunity = personalFacePriceMapper.queryFromFacePriceHis(roadNumberAddress,startDate,facePriceQuery.getAcreageMin(),facePriceQuery.getAcreageMax());
            filterByCityAndDistrict(byCommunityNameResult,targetAddress);
            byCommunityNameResult.stream().forEach(x->x.setMatching("楼盘名匹配"));
            facePriceByCommunity.stream().forEach(x->x.setMatching("楼盘名匹配(口估)"));
        }


        //合并历史口估结果集
        facePricesByRoadNumber.addAll(facePricesByRoadNameNumber);
        facePricesByRoadNumber.addAll(facePriceByCommunity);
        facePricesByRoadNumber = facePricesByRoadNumber.stream().distinct().collect(Collectors.toList());


        //合并查询结果集
        byRoadNumberResult.addAll(byRoadNameNumberResult);
        byRoadNumberResult.addAll(byCommunityNameResult);

        //去重
        byRoadNumberResult = byRoadNumberResult.stream().distinct().collect(Collectors.toList());

        //合并评估和口估
        byRoadNumberResult.addAll(facePricesByRoadNumber);

        //排序
        Collections.sort(byRoadNumberResult);
        return byRoadNumberResult;
    }

    @Override
    public ExternalFacePriceVO externalPriceByLocation(PersonalFacePriceQueryDTO facePriceQuery) {

        Date startDate = DateUtils.facePriceLimitDate(facePriceQuery.getLimit());
        String location = facePriceQuery.getQueryTarget();

        //获取全路名 查询小区名称
        List<String> communityName1 = new ArrayList<>();
                String roadNumberAddress = AddressUtil.getRoadNumberAddress(location);
        if (StrUtil.isNotBlank(roadNumberAddress)){
            communityName1 = personalFacePriceMapper.queryCommunityNameByFuzzyAddress(roadNumberAddress);
        }

        //获取路名路号
        List<String> communityName2 = new ArrayList<>();
                String roadAndRoadNumber = AddressUtil.getRoadAndRoadNumber(location);
        if (StrUtil.isNotBlank(roadAndRoadNumber)){
            communityName2 = personalFacePriceMapper.queryCommunityNameByFuzzyAddress(roadAndRoadNumber);
        }

        //获取小区名字 从地址中获取
        String communityNameFromAddress = AddressUtil.getCommunityNameFromAddress(location);

        communityName1.addAll(communityName2);

        if (StrUtil.isNotBlank(communityNameFromAddress)){
            communityName1.add(communityNameFromAddress);
        }

        List<String> communityNames = communityName1.stream().distinct().collect(Collectors.toList());
        return queryExternalDataByCommunityNames(communityNames,startDate,facePriceQuery.getAcreageMin(),facePriceQuery.getAcreageMax());

    }

    @Override
    public List<DayouPersonalPriceVO> personalPriceByCommunity(PersonalFacePriceQueryDTO facePriceQuery) {
        Date startDate = DateUtils.facePriceLimitDate(facePriceQuery.getLimit());
        String community = facePriceQuery.getQueryTarget();
        List<DayouPersonalPriceVO> result = personalFacePriceMapper.queryByFuzzyCommunityName(community,startDate);
        List<DayouPersonalPriceVO> facePrices = personalFacePriceMapper.queryFromFacePriceHis(community, startDate,facePriceQuery.getAcreageMin(),facePriceQuery.getAcreageMax());
        facePrices.forEach(x->{
            x.setMatching("楼盘名(口估)");
            x.setCommunityName(x.getLocation());
            x.setLocation(null);
        });
        result.addAll(facePrices);
        Collections.sort(result);
        return result;
    }

    @Override
    public ExternalFacePriceVO externalPersonalPriceByCommunity(PersonalFacePriceQueryDTO facePriceQuery) {
        Date startDate = DateUtils.facePriceLimitDate(facePriceQuery.getLimit());
        String communityName = facePriceQuery.getQueryTarget();
        List<String> communityNames = Collections.singletonList(communityName);
        return queryExternalDataByCommunityNames(communityNames,startDate,facePriceQuery.getAcreageMin(),facePriceQuery.getAcreageMax());
    }

    @Override
    public Boolean doFacePrice(PersonalFacePrice facePrice) {
        Long currentUserId = LoginContext.getCurrentUserId();
        facePrice.setUserId(currentUserId);
        facePrice.setValueTiming(LocalDate.now());
        facePrice.setFaceAmount(facePrice.getFaceAcreage().multiply(facePrice.getFacePrice()));
        return this.save(facePrice);
    }

    @Override
    public IPage<PersonalFacePrice> getPage(Page page, PersonalFacePrice facePrice) {
        IPage<PersonalFacePrice> result = personalFacePriceMapper.getPage(page,facePrice);
        return result;
    }

    @Override
    public BigDecimal querySysFacePrice(PersonalFacePriceQueryDTO facePriceQuery) {
        //定义面积区间跨度
        BigDecimal diff = new BigDecimal(10);
        BigDecimal acreage = facePriceQuery.getAcreage();
        if (acreage != null){
            facePriceQuery.setAcreageMax(acreage.add(diff));
            facePriceQuery.setAcreageMin(acreage.subtract(diff));
        }


        //获取数据源集合
        List<DayouPersonalPriceVO> dyPrices = this.personalPriceByLocation(facePriceQuery);
        ExternalFacePriceVO externalFacePriceVO = this.externalPriceByLocation(facePriceQuery);

        //提升查询结果率
        if (CollectionUtil.isEmpty(dyPrices)){
            facePriceQuery.setAcreageMin(null);
            facePriceQuery.setAcreageMax(null);
            dyPrices = this.personalPriceByLocation(facePriceQuery);
        }

        if (externalFacePriceVO==null){
            facePriceQuery.setAcreageMin(null);
            facePriceQuery.setAcreageMax(null);
            externalFacePriceVO = this.externalPriceByLocation(facePriceQuery);
        }

        BigDecimal dealAvgPrice = BigDecimal.ZERO;
        BigDecimal dyAvgPrice = BigDecimal.ZERO;
        int i = 0;
        if (externalFacePriceVO != null){
            //链家成交均价
            dealAvgPrice = externalFacePriceVO.getDealPrice();
            if (dealAvgPrice.compareTo(BigDecimal.ZERO) > 0){
                i+=1;
            }
        }

        if (CollectionUtil.isNotEmpty(dyPrices)){
            //大友评估均价
            double reduce = dyPrices.stream().filter(x->x.getPrice()!=null).map(DayouPersonalPriceVO::getPrice).mapToDouble(BigDecimal::doubleValue).reduce(0, Double::sum);
            dyAvgPrice = BigDecimal.valueOf(reduce).divide(BigDecimal.valueOf(dyPrices.size()==0?1:dyPrices.size()), 0, RoundingMode.HALF_UP);
            if (dyAvgPrice.compareTo(BigDecimal.ZERO) > 0){
                i+=1;
            }
        }


        //综合均价
        return i==0?BigDecimal.ZERO:dealAvgPrice.add(dyAvgPrice).divide(new BigDecimal(i), RoundingMode.HALF_UP);
    }

    @Override
    public String uploadPersonalFacePriceTemp(MultipartFile file,String limit) {
        try {
            List<PersonalFacePriceBatchDTO> batchQuery = ExcelUtil.importExcel(PersonalFacePriceBatchDTO.class,file.getInputStream(),1);
            for (PersonalFacePriceBatchDTO batch : batchQuery) {
                PersonalFacePriceQueryDTO target = new PersonalFacePriceQueryDTO();
                BeanUtil.copyProperties(batch, target);
                target.setLimit(limit);
                batch.setPrice(this.querySysFacePrice(target));
                batch.setAmount(batch.getPrice().multiply(batch.getAcreage()==null?BigDecimal.ZERO:batch.getAcreage()).setScale(2, RoundingMode.HALF_UP));
                batch.setValueTiming(LocalDate.now());
            }
            ExcelPlusUtil<PersonalFacePriceBatchDTO> util = new ExcelPlusUtil<PersonalFacePriceBatchDTO>(PersonalFacePriceBatchDTO.class);
            String url = util.exportExcel2File(batchQuery, "批量询价结果");
            return url;
        } catch (IOException e) {
            ErrorCode.throwBusinessException(ErrorCode.DEFAULTERROR);
        }
        return null;
    }


    /**
     * 移除路名相同，但行政区域不同的元素
     * @param priceList
     */
    private void filterByCityAndDistrict(List<DayouPersonalPriceVO> priceList, AddressDTO targetAddress){
        Iterator<DayouPersonalPriceVO> iterator = priceList.iterator();
        while (iterator.hasNext()) {
            DayouPersonalPriceVO personalPriceVO = iterator.next();
            AddressDTO item = AddressUtil.parseAddress(personalPriceVO.getLocation());
            if (StrUtil.isNotBlank(targetAddress.getCity())){
                String city = item.getCity();
                if (StrUtil.isNotBlank(city) && !targetAddress.getCity().contains(city)){
                    iterator.remove();
                }
            }
            if (StrUtil.isNotBlank(targetAddress.getDistrict())){
                String district = item.getDistrict();
                if (StrUtil.isNotBlank(district) && !targetAddress.getDistrict().contains(district)){
                    iterator.remove();
                }
            }

        }
    }


    /**
     * 用楼盘名称查询外部数据
     * @param communityNames
     * @param startDate
     * @return
     */
    private ExternalFacePriceVO queryExternalDataByCommunityNames(List<String> communityNames, Date startDate, BigDecimal acreageMin,BigDecimal acreageMax){
        if (CollectionUtil.isNotEmpty(communityNames)){
            //成交价
            List<ExternalPersonalVO> lianJiaDeal = externalDyDataBaseService.externalPriceLianJiaDeal(communityNames, startDate,acreageMin,acreageMax);
            //List<ExternalPersonalVO> qiYuDeal = externalDyDataBaseService.externalPriceQiYuDeal(communityNames,startDate);
            //挂牌价
            List<ExternalPersonalVO> qiYuUp = externalDyDataBaseService.externalPriceQiYuUp(communityNames, startDate);

            //时间修正参数、面价修正参数、综合修正参数
            String time = StrUtil.isBlank(globalConfigService.getValue(SYS.getCode(), FACE_PRICE_TIME_PARAM.getCode()))?"1":globalConfigService.getValue(SYS.getCode(), FACE_PRICE_TIME_PARAM.getCode());
            String acreage = StrUtil.isBlank(globalConfigService.getValue(SYS.getCode(), FACE_PRICE_ACREAGE_PARAM.getCode()))?"1":globalConfigService.getValue(SYS.getCode(), FACE_PRICE_ACREAGE_PARAM.getCode());
            String overall = StrUtil.isBlank(globalConfigService.getValue(SYS.getCode(), FACE_PRICE_OVERALL_PARAM.getCode()))?"1":globalConfigService.getValue(SYS.getCode(), FACE_PRICE_OVERALL_PARAM.getCode());

            //修正成交价
            List<BigDecimal> dealPrices = lianJiaDeal.stream().map(ExternalPersonalVO::getPrice).collect(Collectors.toList());
            double reduce = dealPrices.stream().mapToDouble(BigDecimal::doubleValue).reduce(0, Double::sum);
            //成交价均值
            BigDecimal dealPrice = BigDecimal.valueOf(reduce).divide(BigDecimal.valueOf(lianJiaDeal.size()==0?1:lianJiaDeal.size()), 0, RoundingMode.HALF_UP);
            //修正
            BigDecimal xDealPrice = dealPrice.multiply(new BigDecimal(time)).multiply(new BigDecimal(acreage)).multiply(new BigDecimal(overall)).setScale(2, RoundingMode.HALF_UP);

            //lianJiaDeal.addAll(qiYuDeal);

            //修正挂牌价
            List<BigDecimal> upPrices = qiYuUp.stream().map(ExternalPersonalVO::getPrice).collect(Collectors.toList());
            double reduce2 = upPrices.stream().mapToDouble(BigDecimal::doubleValue).reduce(0, Double::sum);
            //成交价均值
            BigDecimal upPrice = BigDecimal.valueOf(reduce2).divide(BigDecimal.valueOf(upPrices.size()==0?1: upPrices.size()), 0, RoundingMode.HALF_UP);
            //修正
            BigDecimal xUplPrice = upPrice.multiply(new BigDecimal(time)).multiply(new BigDecimal(acreage)).multiply(new BigDecimal(overall)).setScale(0, BigDecimal.ROUND_HALF_UP);

            ExternalFacePriceVO facePriceVO = new ExternalFacePriceVO();
            facePriceVO.setDealPrice(xDealPrice);
            facePriceVO.setUpPrice(xUplPrice);
            facePriceVO.setDeals(lianJiaDeal);
            facePriceVO.setUps(qiYuUp);
            facePriceVO.setTimeParam(time);
            facePriceVO.setAcreageParam(acreage);
            facePriceVO.setOverAllParam(overall);
            facePriceVO.setDealAvgPrice(dealPrice);
            facePriceVO.setUpAvgPrice(upPrice);
            return facePriceVO;
        }
        return null;
    }
}
