package com.dayou.service.impl;

import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson.JSON;
import com.dayou.common.BaseEntity;
import com.dayou.dto.NameUrlDTO;
import com.dayou.dto.PersonalSceneDTO;
import com.dayou.dto.TaskRecordDTO;
import com.dayou.entity.Personal;
import com.dayou.entity.PersonalScene;
import com.dayou.mapper.PersonalSceneMapper;
import com.dayou.service.IPersonalSceneService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.dayou.utils.LoginContext;
import org.springframework.stereotype.Service;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.dayou.utils.ExcelUtil;
import org.apache.poi.ss.usermodel.Cell;
import org.apache.poi.ss.usermodel.CellStyle;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.springframework.web.multipart.MultipartFile;
import java.util.List;
import java.util.ArrayList;
import java.util.Map;

import org.springframework.transaction.annotation.Transactional;
import com.dayou.enums.BatchTaskTypeEnum;

/**
 * <p>
 * 个贷实勘表 服务实现类
 * </p>
 *
 * @author wucl
 * @since 2024-03-01
 */
@Service
public class PersonalSceneServiceImpl extends ServiceImpl<PersonalSceneMapper, PersonalScene> implements IPersonalSceneService {


    @Override
    @SuppressWarnings("unchecked")
    public Page<PersonalScene> selectPage(Page page,PersonalScene personalScene){
        return this.page(page, new QueryWrapper<PersonalScene>(personalScene));
    }


    @Override
    public PersonalScene detail(Long personalId){
        PersonalScene ps = this.getOne(new LambdaQueryWrapper<PersonalScene>().eq(PersonalScene::getPersonalId, personalId)
                .eq(BaseEntity::getDeleted, Boolean.FALSE));
        PersonalSceneDTO personalSceneDTO = new PersonalSceneDTO();
        if (ps == null){
            PersonalScene p = new PersonalScene();
            p.setPersonalId(personalId);
            p.setPersonalSceneImage(personalSceneDTO);
            this.save(p);
            return p;
        }

        PersonalSceneDTO personalSceneDTO1 = JSON.parseObject(ps.getImages(), PersonalSceneDTO.class);
        if (personalSceneDTO1 !=null) {
            personalSceneDTO = personalSceneDTO1;
        }
        ps.setPersonalSceneImage(personalSceneDTO);
        ps.setImages(null);
        return ps;
    }

    @Override
    public Boolean add(PersonalScene personalScene){
        personalScene.setUserId(LoginContext.getCurrentUserId());
        return  this.save(personalScene);
    }

    @Override
    public Boolean update(PersonalScene personalScene){
        personalScene.setImages(JSON.toJSONString(personalScene.getPersonalSceneImage()));
        return  this.updateById(personalScene);
    }

    @Override
    public Boolean delete(Long id){
        //逻辑删除
        return this.removeById(id);
    }

    @Override
    public Boolean saveSceneInfo(TaskRecordDTO<PersonalScene> taskRecordDTO) {
        PersonalScene taskData = taskRecordDTO.getTaskData();
        taskData.setUserId(LoginContext.getCurrentUserId());
        return this.saveOrUpdate(taskData);
    }

    @Override
    public void getSceneImagesForDoc(Long personalId, Map<String, Object> textMap) {
        PersonalScene per = this.getOne(new LambdaQueryWrapper<PersonalScene>().eq(PersonalScene::getPersonalId, personalId).eq(BaseEntity::getDeleted, false));
        if (per != null && StrUtil.isNotBlank(per.getImages())) {
            PersonalSceneDTO personalSceneDTO = JSON.parseObject(per.getImages(), PersonalSceneDTO.class);
            textMap.put("customerFullName", personalSceneDTO.getCustomerFullName());
            NameUrlDTO mapLocationImage = personalSceneDTO.getMapLocationImage();
            if (mapLocationImage != null && StrUtil.isNotBlank(mapLocationImage.getUrl())) {
                textMap.put("mapLocation", mapLocationImage.getUrl());
            }
            NameUrlDTO sceneImage1 = personalSceneDTO.getSceneImage1();
            if (sceneImage1 != null && StrUtil.isNotBlank(sceneImage1.getUrl())) {
                textMap.put("sceneImage1", sceneImage1.getUrl());
            }
            NameUrlDTO sceneImage2 = personalSceneDTO.getSceneImage2();
            if (sceneImage2 != null && StrUtil.isNotBlank(sceneImage2.getUrl())) {
                textMap.put("sceneImage2", sceneImage2.getUrl());
            }
            NameUrlDTO sceneImage3 = personalSceneDTO.getSceneImage3();
            if (sceneImage3 != null && StrUtil.isNotBlank(sceneImage3.getUrl())) {
                textMap.put("sceneImage3", sceneImage3.getUrl());
            }
            NameUrlDTO sceneImage4 = personalSceneDTO.getSceneImage4();
            if (sceneImage4 != null && StrUtil.isNotBlank(sceneImage4.getUrl())) {
                textMap.put("sceneImage4", sceneImage4.getUrl());
            }
            NameUrlDTO sceneImage5 = personalSceneDTO.getSceneImage5();
            if (sceneImage5 != null && StrUtil.isNotBlank(sceneImage5.getUrl())) {
                textMap.put("sceneImage5", sceneImage5.getUrl());
            }
            NameUrlDTO sceneImage6 = personalSceneDTO.getSceneImage6();
            if (sceneImage6 != null && StrUtil.isNotBlank(sceneImage6.getUrl())) {
                textMap.put("sceneImage6", sceneImage6.getUrl());
            }
            NameUrlDTO sceneImage7 = personalSceneDTO.getSceneImage7();
            if (sceneImage7 != null && StrUtil.isNotBlank(sceneImage7.getUrl())) {
                textMap.put("sceneImage7", sceneImage7.getUrl());
            }
            NameUrlDTO sceneImage8 = personalSceneDTO.getSceneImage8();
            if (sceneImage8 != null && StrUtil.isNotBlank(sceneImage8.getUrl())) {
                textMap.put("sceneImage8", sceneImage8.getUrl());
            }
            NameUrlDTO sceneImage9 = personalSceneDTO.getSceneImage9();
            if (sceneImage9 != null && StrUtil.isNotBlank(sceneImage9.getUrl())) {
                textMap.put("sceneImage9", sceneImage9.getUrl());
            }
            NameUrlDTO sceneImage10 = personalSceneDTO.getSceneImage10();
            if (sceneImage10 != null && StrUtil.isNotBlank(sceneImage10.getUrl())) {
                textMap.put("sceneImage10", sceneImage10.getUrl());
            }
            NameUrlDTO sceneImage11 = personalSceneDTO.getSceneImage11();
            if (sceneImage11 != null && StrUtil.isNotBlank(sceneImage11.getUrl())) {
                textMap.put("sceneImage11", sceneImage11.getUrl());
            }
            NameUrlDTO sceneImage12 = personalSceneDTO.getSceneImage12();
            if (sceneImage12 != null && StrUtil.isNotBlank(sceneImage12.getUrl())) {
                textMap.put("sceneImage12", sceneImage12.getUrl());
            }
            NameUrlDTO sceneImage13 = personalSceneDTO.getSceneImage13();
            if (sceneImage13 != null && StrUtil.isNotBlank(sceneImage13.getUrl())) {
                textMap.put("sceneImage13", sceneImage13.getUrl());
            }

        }
    }
}
